/*
 * FollowMe.c
 *
 *  Created on: 18.05.2012
 *      Author: cebra
 */
/*****************************************************************************
 *   Copyright (C) 2011 Christian "Cebra" Brandtner, brandtner@brandtner.net *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License.          *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *                                                                           *
 *                                                                           *
 *   Credits to:                                                             *
 *   Holger Buss & Ingo Busker from mikrokopter.de for the MK project + SVN  *
 *                          http://www.mikrokopter.de                        *
 *   Gregor "killagreg" Stobrawa for his version of the MK code              *
 *   Thomas Kaiser "thkais" for the original project. See                    *
 *                          http://www.ft-fanpage.de/mikrokopter/            *
 *                          http://forum.mikrokopter.de/topic-4061-1.html    *
 *   Claas Anders "CaScAdE" Rathje for providing the font and his C-OSD code *
 *                          http://www.mylifesucks.de/oss/c-osd/             *
 *   Harald Bongartz "HaraldB" for providing his Ideas and Code for usibility*
 *****************************************************************************/
//############################################################################
//# HISTORY  followme.c
//#
//# 19.06.2014 OG
//# - erster Prototyp der Follow Me zum Kopter sendet
//# - etliche Aenderungen im Codeaufbau
//#
//# 01.06.2014 OG
//# - chg: FollowMe() - verschiedene Anzeigefunktionen auskommentiert und
//#        als DEPRICATED Klassifiziert wegen Cleanup der alten OSD Screens
//# - chg: FollowMe() - Check bzgl. NC-Hardware entfernt da das bereits durch das
//#        Hauptmenue erledigt wird
//#
//# 13.05.2014 OG
//# - chg: FollowMe() - Variable 'flag' auskommentiert
//#        wegen Warning: variable set but not used
//# - chg: FollowMe() - Variable 'old_FCFlags' auskommentiert
//#        wegen Warning: variable set but not used
//# - chg: FollowMe() - den Bereich in dem FC-Settings geladen werdeb
//#        auskommentiert weil man das a) vorallem nicht benoetigt
//#        und b) die load_setting() so nicht mehr existiert
//#        (der Codebereich kann meines erachtens geloescht werden)
//# - del: verschiedene Verweise auf 'mk_param_struct' entfernt, weil es
//#        das a) nicht mehr gibt und b) hier gar nicht benoetigt wird
//# - chg: FollowMe() - OSD_Timeout() entfernt (weil es das nicht mehr gibt)
//#        und erstmal durch ein PKT_Message_P() ersetzt
//#        * ACHTUNG: UNGETESTET! * (bei Bedarf anpassen, followme hat niedrige Prio)
//# - add: #include "../pkt/pkt.h"
//#
//# 05.05.2013 Cebra
//# - chg: #ifdef USE_FOLLOWME
//#
//############################################################################

#include "../cpu.h"
#include <avr/io.h>
#include <inttypes.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "../main.h"

#ifdef USE_FOLLOWME

#include "../followme/followme.h"
#include "../osd/osd.h"
#include "../lcd/lcd.h"
#include "../timer/timer.h"
#include "../uart/usart.h"
#include "../eeprom/eeprom.h"
#include "../messages.h"
#include "../tracking/tracking.h"
#include "../bluetooth/bluetooth.h"
#include "../setup/setup.h"
#include "../uart/uart1.h"
#include "../mk-data-structs.h"
#include "../pkt/pkt.h"

#define COSD_WASFLYING 4
#define TIMEOUT 200     // 2 sec

// global definitions and global vars
NaviData_t *naviData;
unsigned char Element;
uint16_t heading_home;

// Hier Hhenanzeigefehler Korrigieren
#define AltimeterAdjust 1.5


// Positionen der Anzeigeelemente im Bildschirm
#define OSD_ALTITUDE_CONTROL    1
#define OSD_ALTITUDE                     2
#define OSD_BATTERY_LEVEL               3
#define OSD_CAPACITY                    4
#define OSD_CARE_FREE                   5
#define OSD_COMPASS_DEGREE              6
#define OSD_COMPASS_DIRECTION   7
#define OSD_COMPASS_ROSE                8
#define OSD_CURRENT                             9
#define OSD_FLYING_TIME                 10
#define OSD_GROUND_SPEED                11
#define OSD_HOME_CIRCLE                 12
#define OSD_HOME_DEGREE                 13
#define OSD_HOME_DISTANCE               14
#define OSD_LED1_OUTPUT                 15
#define OSD_LED2_OUTPUT                 16
#define OSD_MANUELL                             17
#define OSD_NAVI_MODE                   18
#define OSD_RC_INTENSITY                19
#define OSD_SATS_IN_USE                 20
#define OSD_STATUS_FLAGS                21
#define OSD_VARIOMETER                  22

// Flags
//uint8_t COSD_FLAGS2 = 0;
//
//GPS_Pos_t last5pos[7];
uint8_t FM_error = 0;


//---------------------
// Waypoint Types
// TODO OG: verschieben nach: mk-data-structs.h
//---------------------
#define POINT_TYPE_INVALID 255
#define POINT_TYPE_WP   0
#define POINT_TYPE_POI  1


//---------------------
// Status 
// GPS_Pos_t
//aus MK source
//
// TODO OG: verschieben nach: mk-data-structs.h
//---------------------
#define INVALID         0x00
#define NEWDATA         0x01
#define PROCESSED       0x02


   
//--------------------------------------------------------------
//--------------------------------------------------------------
void FollowMe( void )
{
    GPS_Pos_t currpos;
    uint8_t tmp_dat;
    uint8_t status;
    uint8_t redraw;
    uint8_t drawmode;
    int8_t  yoffs;
	uint32_t NMEA_GPGGA_counter_old;             // Merker: zaehlt empfangene GPGGA-Pakete
	uint32_t send_followme_counter;

	Point_t FollowMe;


    lcd_cls();

    if( !BT_CheckBTM222() ) 
    {
    	return;
    }


    if( (Config.gps_UsedMac[5] == '0') || (Config.gps_UsedMac[6] == '0') )
    {
        lcd_cls ();
        lcd_printp_at (0, 0, PSTR("keine GPS Maus"), MINVERS);
        lcd_printp_at (0, 1, PSTR("konfiguriert!") , MINVERS);
        _delay_ms(2000);
        return;
    }


    //---------------------
    // 1. BT GPS pruefen
    //
    // zeige Daten der BT GPS Maus
    //---------------------
    lcd_cls ();
    lcdx_printf_at( 0, 0, MINVERS, 0,0 , "%21s", Config.gps_UsedDevName );

    set_BTOn();

    if( Config.BTIsSlave )
    {
        bt_downlink_init();
    }

    lcd_printp_at( 0, 1, PSTR("try to connect"), MNORMAL);
    status = bt_connect( Config.gps_UsedMac );

    if( !status )                                                           // keine Verbindung zum BT-Device
    {
        set_BTOff();
        lcd_printp_at (0, 2, PSTR("Error connecting!"), 0);
        Wait4KeyOK();
        return;
    }
    
    receiveNMEA = true;

    lcd_frect( 0, (8*1), 128, 16, 0);                                       // Line 1-2 clear
    lcd_printp_at(19, 7, PSTR("OK")    , MNORMAL);                          // Keyline
    lcd_printp_at( 0, 1, PSTR("Sat: ") , MNORMAL);                          // Label Line 1 L
    lcd_printp_at(11, 1, PSTR("XErr: "), MNORMAL);                          // Label Line 1 R
    lcd_printp_at( 0, 2, PSTR("Fix: ") , MNORMAL);                          // Label Line 2 L
    lcd_printp_at(11, 2, PSTR("HDOP: "), MNORMAL);                          // Label Line 2 r
    lcd_printp_at( 0, 3, PSTR(" Latitude   Longitude"), MINVERS);           // Label Line 3
    lcd_printp_at( 0, 5, PSTR("Alt: "), MNORMAL);                           // Label Line 6

    bt_rx_ready = 0;
    do
    {
       CheckGPS = true;
       if( !bt_receiveNMEA() )  { receiveNMEA = false; break; }

       Tracking_NMEA();

        //---------------

        write_ndigit_number_u( 5, 1, NMEAsatsInUse, 2, 0,0);                // Line 1 L: Sat-Anzahl
        write_ndigit_number_u(17, 1, bt_rxerror, 4, 0,0);                   // Line 1 R: BT RX Error's

        //---------------

        write_ndigit_number_u( 5, 2, posfix, 2, 0,0);                       // Line 2 L: Fix
        if( HDOP == 0 )                                                     // Line 2 R: HDOP
          lcd_printp_at(16, 2, PSTR("  ---"), 0);
        else
          write_ndigit_number_u_10th(16, 2, HDOP, 4, 0,0);

        //---------------

        writex_gpspos( 0, 4, NMEAlatitude , MNORMAL, 0,0 );                 // Line 4 L: Latitude
        writex_gpspos(12, 4, NMEAlongitude, MNORMAL, 0,0 );                 // Line 4 R: Longitude

        //---------------

        write_ndigit_number_u_10th( 5, 5, NMEAaltitude, 5, 0,0);            // Line 6 L: Altitude
        lcd_printp_at(11, 5, PSTR("m"), 0);

        lcd_puts_at(13, 5, NMEAtime, 0);                                    // Line 6 R: GPS Time
        lcd_printp_at(1, 6, PSTR("wait for Sat Fix"), 0);

    } while (!get_key_press (1 << KEY_ENTER) && !posfix==1 );



    //---------------------
    // 2. Follow Me
    //---------------------
    lcd_cls ();

    SwitchToNC();

    mode = 'O';

    // disable debug...
    //      RS232_request_mk_data (0, 'd', 0);
    tmp_dat = 0;
    SendOutData ('d', ADDRESS_ANY, 1, &tmp_dat, 1);

    // request OSD Data from NC every 100ms
    //      RS232_request_mk_data (1, 'o', 100);
    tmp_dat = 10;
    SendOutData ('o', ADDRESS_NC, 1, &tmp_dat, 1);

	//OSD_active = true;              // bentigt fr Navidata Ausgabe an SV2

    timer     = TIMEOUT;
    abo_timer = ABO_TIMEOUT;


	MKLiPoCells_Init();

	redraw = true;

	NMEA_GPGGA_counter     = 0;
	NMEA_GPGGA_counter_old = 0;
	send_followme_counter  = 0;


    do
    {
    	//-----------------------------------------
    	// Screen redraw
    	//-----------------------------------------
    	if( redraw )
    	{
    		lcdx_printf_center_P( 0, MNORMAL, 1,0, PSTR("FollowMe") );	// Titel: oben, mitte

        	lcd_line(  (6*6-3),  0,  (6*6-3), 11, 1);                   // Linie Vertikal links
        	lcd_line( (15*6+5),  0, (15*6+5), 11, 1);                   // Linie Vertikal rechts
        	lcd_line(        0, 12,      127, 12, 1);                   // Linie Horizontal

        	lcd_line(        0, 39,      127, 39, 1);                   // Linie Horizontal Mitte

        	draw_icon_home( 1, 19);
        	draw_icon_target_round( 1, 48);

			redraw = false;
    	}



    	//-----------------------------------------
    	// neue MK Daten vorhanden
    	//-----------------------------------------
		if( rxd_buffer_locked )		// neue MK Daten
        {
        	timer = TIMEOUT;
            Decode64 ();
            naviData = (NaviData_t *) pRxData;

            FM_error = 0;

            currpos.Latitude  = naviData->CurrentPosition.Latitude;
            currpos.Longitude = naviData->CurrentPosition.Longitude;


            //----------------------------------
            // LiPo Cell Check
            //----------------------------------
			MKLiPoCells_Check();					// ggf. Zellenzahl ermitteln


			//#################
    		//# MK
			//#################

    		//-----------------
    		// Oben: MK-Batt Level (Volt)
    		//-----------------
    		OSD_Element_BattLevel2( 0, 0, 0,0 );

    		//-----------------
    		// Oben: Batt Level Bar
    		//-----------------
    		OSD_Element_Battery_Bar( 0, 9, 30, 1, ORIENTATION_H);

    		//-----------------
    		// Oben: MK-Flugzeit
    		//-----------------
    		writex_time(16, 0, naviData->FlyingTime, MNORMAL, 2,0);


    		//-----------------
    		// MK: Sat Anzahl
    		//-----------------
    		//naviData->SatsInUse = 10;
    		yoffs = -26;
    		writex_ndigit_number_u( 17, 5, naviData->SatsInUse, 2, 0,MNORMAL,  3,2+yoffs);
    		draw_icon_satmini( 116+3, 42+yoffs);


    		//-----------------
    		// MK
    		//-----------------
            writex_gpspos( 3, 4, currpos.Latitude , MNORMAL, 1,-5 );                 // Line 4 L: Latitude
            writex_gpspos(12, 4, currpos.Longitude, MNORMAL, 1,-5 );                 // Line 4 R: Longitude



			//#################
    		//# PKT (GPS Maus)
			//#################

    		//-----------------
    		// Receive Counter
    		//-----------------
        	//lcdx_printf_at_P(11, 5, MNORMAL, 0,5, PSTR("R:%4ld"), NMEA_GPGGA_counter );               // Anzahl empfangener NMEA GPGGA-Pakete
        	
    		//-----------------
    		// Send Counter
    		//-----------------
          //lcdx_printf_at_P(4, 5, MNORMAL, 0,5, PSTR("S: %ld"), send_followme_counter );               // Anzahl gesendeter Target-Positionen
        	lcdx_printf_at_P(12, 5, MNORMAL, 1,5, PSTR("%5ld"), send_followme_counter );               // Anzahl gesendeter Target-Positionen

    		//-----------------
    		// PKT: Sat Anzahl
    		//-----------------
            if( NMEAsatsInUse > 5 )	drawmode = MNORMAL;
            else 					drawmode = MINVERS;
    		yoffs = 3;
    		writex_ndigit_number_u( 17, 5, NMEAsatsInUse, 2, 0,drawmode,  4,2+yoffs);
    		draw_icon_satmini( 116+4, 42+yoffs);


    		//-----------------
			// PKT Lat / Long
    		//-----------------
            writex_gpspos( 3, 7, NMEAlatitude , MNORMAL, 1,0 );                 // Line 4 L: Latitude
            writex_gpspos(12, 7, NMEAlongitude, MNORMAL, 1,0 );                 // Line 4 R: Longitude
			

            rxd_buffer_locked = FALSE;
        }
 

    	//-----------------------------------------
    	//-----------------------------------------
        if( !abo_timer )
        {
        	// renew abo every 3 sec
            // request OSD Data from NC every 100ms
            //      RS232_request_mk_data (1, 'o', 100);
            tmp_dat = 10;
            SendOutData ( 'o', ADDRESS_NC, 1, &tmp_dat, 1);

            abo_timer = ABO_TIMEOUT;
        }


    	//-----------------------------------------
    	//-----------------------------------------
        if( !timer )
        {
        	//PKT_Message_P( *text, error, timeout, beep, clearscreen )
            PKT_Message_P( strGet(OSD_ERROR), true, 200, true, true );   // max. 2 Sekunden anzeigen

            FM_error = 1;
        }


    	//-----------------------------------------
    	//-----------------------------------------
        if( !bt_receiveNMEA() )  
        { 
        	receiveNMEA = false;
        }
        else  
        { 
        	Tracking_NMEA();
        }


    	//-----------------------------------------
    	// neue NMEA Daten?
    	//-----------------------------------------
    	if( NMEA_GPGGA_counter > NMEA_GPGGA_counter_old ) 
    	{
    		if( (NMEAsatsInUse > 5) && (posfix > 0) )
    		{
				//Config.FM_Refresh

				/*
                pGPSPosTgt->Longitude   = pGPSPosSrc->Longitude;
                pGPSPosTgt->Latitude    = pGPSPosSrc->Latitude;
                pGPSPosTgt->Altitude    = pGPSPosSrc->Altitude;
                pGPSPosTgt->Status              = NEWDATA; // mark data in target position as new
                */

                FollowMe.Position.Status	= NEWDATA;
                FollowMe.Position.Longitude	= NMEAlongitude;
                FollowMe.Position.Latitude	= NMEAlatitude;
              //FollowMe.Position.Altitude	= 0;
                FollowMe.Position.Altitude	= NMEAaltitude;

            	FollowMe.Heading = -1;          					// invalid heading
                FollowMe.ToleranceRadius = Config.FM_Radius;   		// 5 meter default
                FollowMe.HoldTime = 30;          					// ????? go home after 60s without any update ??????
                FollowMe.Event_Flag = 0;        					// no event
                FollowMe.Index = 1;             					// 2st wp
                FollowMe.Type = POINT_TYPE_WP;  					// Typ des Wegpunktes                     
                FollowMe.Name[0] = 'F';        						// Name des Wegpunktes (ASCII)
                FollowMe.Name[1] = 'o';
                FollowMe.Name[2] = 'M';
                FollowMe.Name[3] = 'e';
                FollowMe.WP_EventChannelValue = 0;  				// Will be transferred to the FC and can be used as Poti value there
                FollowMe.AltitudeRate = 0;          				// rate to change the setpoint
                FollowMe.Speed = Config.FM_Speed;                	// rate to change the Position
                FollowMe.CamAngle = 255;            				// Camera servo angle in degree (255 -> POI-Automatic)
                FollowMe.reserve[0] = 0;                			// reserve
                FollowMe.reserve[1] = 0;                			// reserve

        		SendOutData( 's', ADDRESS_NC, 1, &FollowMe, sizeof(FollowMe) );       //'s' = target Position  'w' = Waypoint
        		send_followme_counter++;

				//void SendOutData(uint8_t cmd, uint8_t addr, uint8_t numofbuffers, ...) // uint8_t *pdata, uint8_t len, ...
				// SendOutData ('d', ADDRESS_ANY, 1, &tmp_dat, 1); 
                //SendOutData( 's', NC_ADDRESS, 1, (uint8_t *)&FollowMe, sizeof(FollowMe));   //'s' = target Position  'w' = Waypoint

    		}
    		
    		NMEA_GPGGA_counter_old = NMEA_GPGGA_counter;
    	}


    	//-----------------------------------------
    	// TASTEN
    	//-----------------------------------------
    	if( get_key_press(1 << KEY_ESC) )
    	{
    		break;
    	}

    	if( get_key_press(1 << KEY_ENTER) )
    	{
    		break;
    	}

	} while( (receiveNMEA) );



    //---------------------
    // BEENDEN
    //---------------------
    OSD_active = false;

    //---------------------
    // GPS beenden
    //---------------------
    lcd_cls();
    CheckGPS = false;

    if( !receiveNMEA )
    {
        lcd_printp_at( 0, 1, PSTR("lost BT data"), MNORMAL );
        set_beep( 400, 0x0080, BeepNormal);
    }
    lcd_printp_at( 0, 3, PSTR("GPS trennen")    , MNORMAL );
    lcd_printp_at( 0, 4, PSTR("bitte warten..."), MNORMAL );

    receiveNMEA = false;
    CheckGPS    = false;

    bt_disconnect();

    set_BTOff();
}







//--------------------------------------------------------------
//--------------------------------------------------------------
void FollowMe_OLD( void )
{
        //uint8_t flag;
        uint8_t tmp_dat;
        uint8_t status;
        // flags from last round to check for changes
        //uint8_t old_FCFlags = 0;

        lcd_cls();

        if( !BT_CheckBTM222() ) return;

        if( (Config.gps_UsedMac[5] == '0') || (Config.gps_UsedMac[6] == '0') )
        {
            lcd_cls ();
            lcd_printp_at (0, 0, PSTR("keine GPS Maus"), MINVERS);
            lcd_printp_at (0, 1, PSTR("konfiguriert!") , MINVERS);
            _delay_ms(2000);
            return;
        }

        lcd_cls ();
        lcdx_printf_at( 0, 0, MINVERS, 0,0 , "%21s", Config.gps_UsedDevName );

        set_BTOn();

        if( Config.BTIsSlave )
        {
            bt_downlink_init();
        }

        lcd_printp_at( 0, 1, PSTR("try to connect"), MNORMAL);
        status = bt_connect( Config.gps_UsedMac );

        if( !status )                                                           // keine Verbindung zum BT-Device
        {
            set_BTOff();
            lcd_printp_at (0, 2, PSTR("Error connecting!"), 0);
            Wait4KeyOK();
            return;
        }
        //---------------------
        // 1. BT GPS pruefen
        //
        // zeige Daten der BT GPS Maus
        //---------------------
        receiveNMEA = true;

        lcd_frect( 0, (8*1), 128, 16, 0);                                       // Line 1-2 clear
        lcd_printp_at(19, 7, PSTR("OK")    , MNORMAL);                          // Keyline
        lcd_printp_at( 0, 1, PSTR("Sat: ") , MNORMAL);                          // Label Line 1 L
        lcd_printp_at(11, 1, PSTR("XErr: "), MNORMAL);                          // Label Line 1 R
        lcd_printp_at( 0, 2, PSTR("Fix: ") , MNORMAL);                          // Label Line 2 L
        lcd_printp_at(11, 2, PSTR("HDOP: "), MNORMAL);                          // Label Line 2 r
        lcd_printp_at( 0, 3, PSTR(" Latitude   Longitude"), MINVERS);           // Label Line 3
        lcd_printp_at( 0, 5, PSTR("Alt: "), MNORMAL);                           // Label Line 6

        bt_rx_ready = 0;
        do
        {
           CheckGPS = true;
           if( !bt_receiveNMEA() )  { receiveNMEA = false; break; }

           Tracking_NMEA();

            //---------------

            write_ndigit_number_u( 5, 1, NMEAsatsInUse, 2, 0,0);                // Line 1 L: Sat-Anzahl
            write_ndigit_number_u(17, 1, bt_rxerror, 4, 0,0);                   // Line 1 R: BT RX Error's

            //---------------

            write_ndigit_number_u( 5, 2, posfix, 2, 0,0);                       // Line 2 L: Fix
            if( HDOP == 0 )                                                     // Line 2 R: HDOP
              lcd_printp_at(16, 2, PSTR("  ---"), 0);
            else
              write_ndigit_number_u_10th(16, 2, HDOP, 4, 0,0);

            //---------------

            writex_gpspos( 0, 4, NMEAlatitude , MNORMAL, 0,0 );                 // Line 4 L: Latitude
            writex_gpspos(12, 4, NMEAlongitude, MNORMAL, 0,0 );                 // Line 4 R: Longitude

            //---------------

            write_ndigit_number_u_10th( 5, 5, NMEAaltitude, 5, 0,0);            // Line 6 L: Altitude
            lcd_printp_at(11, 5, PSTR("m"), 0);

            lcd_puts_at(13, 5, NMEAtime, 0);                                    // Line 6 R: GPS Time
            lcd_printp_at(1, 6, PSTR("wait for Sat Fix"), 0);

        } while (!get_key_press (1 << KEY_ENTER) && !posfix==1 );





        //---------------------
        // 2. Follow Me
        //---------------------
        lcd_cls ();

        SwitchToNC();

        mode = 'O';

        // disable debug...
        //      RS232_request_mk_data (0, 'd', 0);
        tmp_dat = 0;
        SendOutData ('d', ADDRESS_ANY, 1, &tmp_dat, 1);

        // request OSD Data from NC every 100ms
        //      RS232_request_mk_data (1, 'o', 100);
        tmp_dat = 10;
//        OSD_active = true;              // bentigt fr Navidata Ausgabe an SV2
        SendOutData ('o', ADDRESS_NC, 1, &tmp_dat, 1);

        //flag = 0;
        timer = TIMEOUT;
        abo_timer = ABO_TIMEOUT;

        do
        {
                if (rxd_buffer_locked)
                {
                        timer = TIMEOUT;
                        Decode64 ();
                        naviData = (NaviData_t *) pRxData;

                        if(FM_error == 1)
                                lcd_cls();

                        FM_error = 0;
                        GPS_Pos_t currpos;
                        currpos.Latitude = naviData->CurrentPosition.Latitude;
                        currpos.Longitude = naviData->CurrentPosition.Longitude;


                        //flag = 1;


                        {
                                //-------------------------------------------
                                // OSD_Screen_Element() is depricated!
                                // USE NEW OSD_Element_xyz() FUNCTIONS!
                                //-------------------------------------------

                                // Altitude Control
                                //OSD_Screen_Element (0, 3, OSD_ALTITUDE_CONTROL,Config.OSD_ScreenMode);
                                //---
                                // 01.06.2014 OG: DEPRICATED  (alte Funktionen in osd/osdold_screens.c - nicht mehr verwenden!)
                                //---
                                //OSD_Element_AltitudeControl( 0, 0 );


                                // Battery level
                                //OSD_Screen_Element (0, 7, OSD_BATTERY_LEVEL,Config.OSD_ScreenMode);
                                //OSD_Element_BatteryLevel( 0, 7, Config.OSD_ScreenMode );

                                // Capacity
                                //OSD_Screen_Element (13, 7, OSD_CAPACITY,Config.OSD_ScreenMode);
                                //OSD_Element_Capacity( 13, 7 );

                                // Care Free
                                //OSD_Screen_Element (12, 2, OSD_CARE_FREE,Config.OSD_ScreenMode);
                                //---
                                // 01.06.2014 OG: DEPRICATED  (alte Funktionen in osd/osdold_screens.c - nicht mehr verwenden!)
                                //---
                                //OSD_Element_CareFree( 12, 2 );

                                // Current
                                //OSD_Screen_Element (7, 7, OSD_CURRENT,Config.OSD_ScreenMode);
                                //OSD_Element_Current( 7, 7 );


                                // Flying time
                                //lcd_printp_at (0, 1, PSTR("Flytime:"), 0);
                                //OSD_Screen_Element (8, 1, OSD_FLYING_TIME,Config.OSD_ScreenMode);
                                //OSD_Element_FlyingTime( 8, 1 );


                                // Ground Speed
                                //lcd_printp_at (0, 0, PSTR("Speed:"), 0);
                                //OSD_Screen_Element (6, 0, OSD_GROUND_SPEED,Config.OSD_ScreenMode);
                                //OSD_Element_GroundSpeed( 6, 0 );

                                // Sats in use
                                //OSD_Screen_Element (16, 0, OSD_SATS_IN_USE,Config.OSD_ScreenMode);
                                //OSD_Element_SatsInUse( 16, 0, Config.OSD_ScreenMode );

								//// Home Distance
								//lcd_printp_at (11, 3, PSTR("Dist:"), 0);
								////OSD_Screen_Element (17, 3, OSD_HOME_DISTANCE,Config.OSD_ScreenMode);
								//OSD_Element_HomeDistance( 17, 3, Config.OSD_ScreenMode );

                                // Navi Mode
                                //OSD_Screen_Element (0, 2, OSD_NAVI_MODE,Config.OSD_ScreenMode);
                                //---
                                // 01.06.2014 OG: DEPRICATED  (alte Funktionen in osd/osdold_screens.c - nicht mehr verwenden!)
                                //---
                                //OSD_Element_NaviMode( 0, 2 );


                                lcd_printp_at (0, 4, PSTR("MK:"), 0);
                                lcd_printp_at (0, 5, PSTR("BT:"), 0);
								//write_ndigit_number_u (3, 4, (uint16_t)(currpos.Latitude/10000000), 2, 0,0);
								//lcd_printp_at (5, 4, PSTR("."), 0);
								//write_ndigit_number_u (6, 4, (uint16_t)((currpos.Latitude/1000) % 10000), 4, 1,0);
								//write_ndigit_number_u (10, 4, (uint16_t)((currpos.Latitude/10) % 100), 2, 1,0);
								//write_gps_pos( 0, 4, currpos.Latitude);

								//write_ndigit_number_u (12, 4, (uint16_t)(currpos.Longitude/10000000), 2, 0,0);
								//lcd_printp_at (14, 4, PSTR("."), 0);
								//write_ndigit_number_u (15, 4, (uint16_t)((currpos.Longitude/1000) % 10000), 4, 1,0);
								//write_ndigit_number_u (19, 4, (uint16_t)((currpos.Longitude/10) % 100), 2, 1,0);
								//write_gps_pos( 12, 4, currpos.Longitude);


                          		writex_gpspos( 3, 4, currpos.Latitude, MNORMAL, 0,0 );                 // Line 4 L: Latitude
                                writex_gpspos(12, 4, currpos.Longitude, MNORMAL, 0,0 );                 // Line 4 R: Longitude
                                if (NMEAsatsInUse>5)
                                  {
                                    writex_gpspos( 3, 5, NMEAlatitude , MNORMAL, 0,0 );                 // Line 4 L: Latitude
                                    writex_gpspos(12, 5, NMEAlongitude, MNORMAL, 0,0 );                 // Line 4 R: Longitude
                                  }
                                else
                                  {
                                    writex_gpspos( 3, 5, NMEAlatitude , MINVERS, 0,0 );                 // Line 4 L: Latitude
                                    writex_gpspos(12, 5, NMEAlongitude, MINVERS, 0,0 );                 // Line 4 R: Longitude
                                  }


//                                lcd_printp_at (0, 5, PSTR("!!!not implemented!!!"), 2);

                                // Akku Warnung
                                if (naviData->UBat < Config.MK_LowBat)
                                { //Beeper ein
                                        set_beep ( 3000, 0x0020, BeepNormal);
                                }
                                if (naviData->UBat > Config.MK_LowBat)  //bei kurzzeitigen Schwankungen Beeper erst wieder aus wenn UBat 0,2 V hher als Warnschwelle
                                { //Beeper aus
                                    set_beep ( 0, 0, BeepOff);
                                }
//                               Akku Warnung Ende

                                //old_FCFlags = naviData->FCStatusFlags;

                                rxd_buffer_locked = FALSE;
                        }


                        if (!abo_timer)
                        {       // renew abo every 3 sec
                                // request OSD Data from NC every 100ms
                                //      RS232_request_mk_data (1, 'o', 100);
                                tmp_dat = 10;
                                SendOutData ('o', ADDRESS_NC, 1, &tmp_dat, 1);

                                abo_timer = ABO_TIMEOUT;
                        }
                }
                if (!timer)
                {
                        //OSD_Timeout(flag);  // <- 13.05.2014 OG: gibt es nicht mehr - ersetzt durch PKT_Message_P() (UNGETESTET!)

                        //void PKT_Message_P( const char *text, uint8_t error, uint16_t timeout, uint8_t beep, uint8_t clearscreen )
                        PKT_Message_P( strGet(OSD_ERROR), true, 200, true, true );   // max. 2 Sekunden anzeigen

                        //flag = 0;
                        FM_error = 1;
                }
                if( !bt_receiveNMEA() )  { receiveNMEA = false;}
                else  Tracking_NMEA();


        }

        while (!get_key_press (1 << KEY_ESC)||(!receiveNMEA) );


        OSD_active = false;
        //---------------------
        // GPS beenden
        //---------------------
        lcd_cls();
        CheckGPS = false;

        if( !receiveNMEA )
        {
            lcd_printp_at( 0, 1, PSTR("lost BT data"), MNORMAL );
            set_beep( 400, 0x0080, BeepNormal);
        }
        lcd_printp_at( 0, 3, PSTR("GPS trennen")    , MNORMAL );
        lcd_printp_at( 0, 4, PSTR("bitte warten..."), MNORMAL );

        receiveNMEA = false;
        CheckGPS    = false;

        bt_disconnect();

        set_BTOff();
}












#endif


