/*****************************************************************************
 *   Copyright (C) 2008 Thomas Kaiser, thomas@ft-fanpage.de                  *
 *   Copyright (C) 2009 Peter "woggle" Mack, mac@denich.net                  *
 *   Copyright (C) 2011 Christian "Cebra" Brandtner, brandtner@brandtner.net *
 *   Copyright (C) 2011 Harald Bongartz                                      *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License.          *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *                                                                           *
 *                                                                           *
 *   Credits to:                                                             *
 *   Holger Buss & Ingo Busker from mikrokopter.de for the MK project + SVN  *
 *                          http://www.mikrokopter.de                        *
 *   Gregor "killagreg" Stobrawa for his version of the MK code              *
 *   Thomas Kaiser "thkais" for the original project. See                    *
 *                          http://www.ft-fanpage.de/mikrokopter/            *
 *                          http://forum.mikrokopter.de/topic-4061-1.html    *
 *   Claas Anders "CaScAdE" Rathje for providing the font and his C-OSD code *
 *                          http://www.mylifesucks.de/oss/c-osd/             *
 *   Harald Bongartz "HaraldB" for providing his Ideas and Code for usibility*
 *****************************************************************************/

//############################################################################
//# HISTORY  mkdisplay.c
//#
//# 29.01.2014 OG
//# - chg: ehemals display_data() jetzt MK_display()
//# - chg: ehemals 'display.c' jetzt 'mk/mkdisplay.c'
//#
//# 25.01.2014 OG
//# - optisches Facelift
//# - PKT_CtrlHook() eingeklinkt
//# - Versionshistorie hinzugefuegt
//############################################################################


#include "../cpu.h"
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdbool.h>

#include "../main.h"
#include "../lcd/lcd.h"
#include "../uart/usart.h"
#include "../timer/timer.h"
#include "../messages.h"
#include "../lipo/lipo.h"
#include "../pkt/pkt.h"
#include "../mk-data-structs.h"

#define TIMEOUT 500	// 5 sec


//--------------------------------------------------------------
//--------------------------------------------------------------
void MK_Display(void)
{
	uint8_t cmd;
	uint8_t flag = 0;
	uint8_t redraw;
	char    c;

    mode              = 'H';
	rxd_buffer_locked = FALSE;
	timer             = TIMEOUT;
	cmd               = 0xfc;	// Home = first page

    redraw = true;

	do
	{
        //------------------------------------------
        // Screen neu zeichnen
        //------------------------------------------
		if( redraw )
		{
	    	lcd_cls ();
        	lcd_frect( 0, 0, 127, 8, 1);      					// Headline: Box fill Black

    		#ifdef HWVERSION3_9
            	lcd_frect( (17*6),  0, (4*6), 8, 0);            // Headline: clear ganz rechts fuer LiPo Anzeige
    		#endif

        	lcd_rect( 0, 2*7-3, 127, 5*7+3+3, 1);               // Rahmen	

	    	lcd_printp_at (0, 0, PSTR("    Display "), 2);

	    	lcd_printp_at( 0, 7, strGet(KEYLINE3), 0);			// Keyline
	
	    	if( current_hardware == NC )
	    	{
	    		lcd_printp_at(  1, 0, PSTR("NC"), MINVERS);
	    		lcd_printp_at( 19, 7, PSTR("FC"), MNORMAL);
	    	}
	    	else
	    	{
	    		if( hardware == FC )
	    		{
  	    		    lcd_printp_at(  1, 0, PSTR("FC"), MINVERS);
	    			lcd_printp_at (19, 7, PSTR("  "), MNORMAL);
	    		}
	    		else
	    		{
  	    		    lcd_printp_at(  1, 0, PSTR("FC"), MINVERS);
	    			lcd_printp_at (19, 7, PSTR("NC"), MNORMAL);
	    		}
	    	}
	    	
	    	redraw = false;
		}
		
		
    	#ifdef HWVERSION3_9
			show_Lipo();
    	#endif
		

		SendOutData('h', ADDRESS_ANY, 1, &cmd, 1);
		cmd = 0xff;
		
		_delay_ms(40);

		if( rxd_buffer_locked )
		{
			Decode64 ();
			flag = 1;

			if (!hardware)
			{	
				// hardware was not detected at startup
				hardware = rxd_buffer[1] - 'a';
				if (hardware == NC)
				{
					current_hardware = NC;
				}
				else
				{
					current_hardware = FC;
				}
				redraw = true;
			}


            //------------------------------------------
            // Ausgabe auf PKT-Anzeige
            // 4 Zeilen a 20 Zeichen
            //------------------------------------------
            c = rxd_buffer[23];
            rxd_buffer[23] = 0;
			lcdx_print_at( 0,2, (uint8_t *) &rxd_buffer[3+ 0],  MNORMAL,  5,-1);
            rxd_buffer[23] = c;

            c = rxd_buffer[43];
            rxd_buffer[43] = 0;
			lcdx_print_at( 0,3, (uint8_t *) &rxd_buffer[3+20],  MNORMAL,  5,0);
            rxd_buffer[43] = c;

            c = rxd_buffer[63];
            rxd_buffer[63] = 0;
			lcdx_print_at( 0,4, (uint8_t *) &rxd_buffer[3+40],  MNORMAL,  5,1);
            rxd_buffer[63] = c;

            rxd_buffer[83] = 0;
			lcdx_print_at( 0,5, (uint8_t *) &rxd_buffer[3+60],  MNORMAL,  5,2);


            //------------------------------------------
            // RX-Buffer freigeben
            //------------------------------------------
			rxd_buffer_locked = FALSE;
			timer = TIMEOUT;
			
		} // end: if( rxd_buffer_locked )


        //------------------------------------------
        // Pruefe PKT Update oder andere PKT-Aktion
        //------------------------------------------
        if( PKT_CtrlHook() )                                                                // Update vom Updatetool angefordert?
        {
			redraw = true;
        }


		if (get_key_press (1 << KEY_MINUS) || get_key_long_rpt_sp ((1 << KEY_MINUS), 2))
		{
			cmd = 0xfe;	// next page
			//SendOutData('h', ADDRESS_ANY, 1, &cmd, 1);
			//cmd = 0;
		}
		else if (get_key_press (1 << KEY_PLUS) || get_key_long_rpt_sp ((1 << KEY_PLUS), 2))
		{
			cmd = 0xfd;	// previous page
			//SendOutData('h', ADDRESS_ANY, 1, &cmd, 1);
			//cmd = 0;
		}
		else if ((hardware == NC) && get_key_press (1 << KEY_ENTER))
		{
			if (current_hardware == NC)
			{
				SwitchToFC();
				redraw = true;
				//timer = TIMEOUT;
			}
			else
			{
				SwitchToNC();
				redraw = true;
				//timer = TIMEOUT;
			}
			cmd = 0xfc;	// Home = first page
			//SendOutData('h', ADDRESS_ANY, 1, &cmd, 1);
			//cmd = 0;
		}
	}
	while (!get_key_press (1 << KEY_ESC) && timer);
		get_key_press(KEY_ALL);

	mode = 0;
	rxd_buffer_locked = FALSE;

	if (!timer)
	{	// timeout occured
		if (flag)
		{
			lcd_cls ();
		}
		lcd_printp_at (0, 2, PSTR("Fehler: Keine Daten"), 0);

		timer = 100;
		while (timer > 0);
	}
	SwitchToNC();
}
