/*
 * wifly_setup.c
 *
 *  Created on: 02.07.2013
 *      Author: cebra
 */
/*****************************************************************************
 *   Copyright (C) 2008 Thomas Kaiser, thomas@ft-fanpage.de                  *
 *   Copyright (C) 2009 Peter "woggle" Mack, mac@denich.net                  *
 *   Copyright (C) 2011 Christian "Cebra" Brandtner, brandtner@brandtner.net *
 *   Copyright (C) 2011 Harald Bongartz                                      *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License.          *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *                                                                           *
 *                                                                           *
 *   Credits to:                                                             *
 *   Holger Buss & Ingo Busker from mikrokopter.de for the MK project + SVN  *
 *                        http://www.mikrokopter.de                          *
 *   Gregor "killagreg" Stobrawa for his version of the MK code              *
 *   Thomas Kaiser "thkais" for the original project. See                    *
 *                        http://www.ft-fanpage.de/mikrokopter/              *
 *                        http://forum.mikrokopter.de/topic-4061-1.html      *
 *   Claas Anders "CaScAdE" Rathje for providing the font and his C-OSD code *
 *                        http://www.mylifesucks.de/oss/c-osd/               *
 *   Harald Bongartz "HaraldB" for providing his Ideas and Code for usibility*
 *****************************************************************************/

//############################################################################
//# HISTORY  wifly_setup.c
//#
//# noch UNVOLLSTNDIG!! (Cebra)
//#
//# 30.03.2014 OG
//# - chg: Text von WL_INSTALLED geaendert von "Modul eingebaut?" auf "Modul vorhanden?"
//#        -> das Modul wird aufgesteckt und nicht eingebaut...
//# - chg: ein paar englische Texte geaendert
//# - chg: Sprache Hollaendisch vollstaendig entfernt
//# - chg: MenuCtrl_PushML_P() umgestellt auf MenuCtrl_PushML2_P()
//#
//# 11.03.2014 SH
//# - add: neue Menpunkte Reset, Version und Update
//# - chg: Menpunkt Passwort wurde wieder aktiviert
//# - chg: beim Menpunkt Kanal fngt die Auswahl jetzt bei 0 an (Modul sucht sich automatisch den besten Kanal)
//#
//# 12.02.2014 OG
//# - chg: Setup_WiFly() Auskommentiert: "unused variable 'z'"
//# - chg: Setup_WiFly() Auskommentiert: "unused variable 'i'"
//#
//# 02.07.2013 cebra
//# - new: wifly_setup(): Setup fr WiFly WLAN-Modul
//###########################################################################


#include "../cpu.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <util/delay.h>
#include "../main.h"
#include "../lcd/lcd.h"
#include "../messages.h"
#include "../utils/menuctrl.h"
#include "../utils/xutils.h"
#include "../eeprom/eeprom.h"
#include "../uart/uart1.h"
#include "../setup/setup.h"
#include "../wifly/PKT_WiFlyHQ.h"

#ifdef USE_WLAN

//-----------------------------
// Setup_WiFly() (WLAN)
//-----------------------------
#define WL_INSTALLED  1
#define WL_SSID       2
#define WL_PASSWORD   3
#define WL_CHANNEL    4
#define WL_INIT       5
#define WL_RESET      6
#define WL_VERSION	  7
#define WL_UPDATE	  8
#define WL_PCCONFIG   9
#define WL_WPA        10


char buf[80];

static const char WL_INSTALLED_de[]		PROGMEM = "Modul vorhanden?";
static const char WL_INSTALLED_en[]     PROGMEM = "Module existent?";
                                                          
static const char WL_SSID_de[]          PROGMEM = "SSID"; 
#define           WL_SSID_en            WL_SSID_de        
                                                          
static const char WL_PASSWORD_de[]      PROGMEM = "Passwort";
static const char WL_PASSWORD_en[]      PROGMEM = "Password";

static const char WL_CHANNEL_de[]       PROGMEM = "Kanal";
static const char WL_CHANNEL_en[]       PROGMEM = "Channel";

static const char WL_INIT_de[]          PROGMEM = "Initialisieren";
static const char WL_INIT_en[]          PROGMEM = "Initialize";

static const char WL_VERSION_de[]       PROGMEM = "Version";
#define			  WL_VERSION_en			WL_VERSION_de

static const char WL_RESET_de[]         PROGMEM = "Reset";
#define			  WL_RESET_en			WL_RESET_de

static const char WL_UPDATE_de[]		PROGMEM = "Update";
#define			  WL_UPDATE_en			WL_UPDATE_de

static const char WL_PCCONFIG_de[]      PROGMEM = "Konfig. mit PC";
static const char WL_PCCONFIG_en[]      PROGMEM = "Config. with PC";

static const char WL_WPA_de[]           PROGMEM = "Sicherheit";
static const char WL_WPA_en[]           PROGMEM = "Security";

//############################################################################
//--------------------------------------------------------------
// Setup_WiFly_MenuCreate()
//
// das Menue aendert sich je nachdem ob WiFly ein- oder
// ausgeschaltet ist
//--------------------------------------------------------------
void Setup_WiFly_MenuCreate( void )
{
    //---------------
    // Create
    //---------------
    MenuCtrl_Create();
    MenuCtrl_SetTitle_P( PSTR("WLAN Setup") );

    //---------------
    // Menuitems
    //---------------
    MenuCtrl_PushML2_P( WL_INSTALLED , MENU_ITEM, NOFUNC , WL_INSTALLED_de , WL_INSTALLED_en );

    if( Config.UseWL )
    {
        MenuCtrl_PushML2_P( WL_SSID      , MENU_ITEM, NOFUNC          , WL_SSID_de      , WL_SSID_en     );
        MenuCtrl_PushML2_P( WL_PASSWORD  , MENU_ITEM, NOFUNC          , WL_PASSWORD_de  , WL_PASSWORD_en );
        MenuCtrl_PushML2_P( WL_CHANNEL   , MENU_ITEM, NOFUNC          , WL_CHANNEL_de   , WL_CHANNEL_en  );
        MenuCtrl_PushML2_P( WL_INIT      , MENU_ITEM, NOFUNC          , WL_INIT_de      , WL_INIT_en     );
        MenuCtrl_PushML2_P( WL_RESET     , MENU_ITEM, NOFUNC          , WL_RESET_de     , WL_RESET_en    );
		MenuCtrl_PushML2_P( WL_VERSION   , MENU_ITEM, NOFUNC          , WL_VERSION_de   , WL_VERSION_en  );
		MenuCtrl_PushML2_P( WL_UPDATE    , MENU_ITEM, NOFUNC          , WL_UPDATE_de    , WL_UPDATE_en   );
		MenuCtrl_PushML2_P( WL_PCCONFIG  , MENU_ITEM, &Port_FC2CFG_WL , WL_PCCONFIG_de  , WL_PCCONFIG_en );
      // MenuCtrl_PushML2_P( WL_WPA      , MENU_ITEM, NOFUNC          , WL_WPA_de       , WL_WPA_en      );
    }
}


//--------------------------------------------------------------
// Setup_WiFly()
//--------------------------------------------------------------
void Setup_WiFly( void )
{
    uint8_t itemid;
    uint8_t UseWL;
    //uint8_t i;
    //uint8_t z;	
    char string[11];

    Setup_WiFly_MenuCreate();

    //---------------
    // Control
    //---------------
    while( true )
    {
        MenuCtrl_Control( MENUCTRL_EVENT );

        if( MenuCtrl_GetKey() == KEY_ESC )  break;      // ENDE

        itemid = MenuCtrl_GetItemId();                  // welcher Menu-Punkt (ID)
        edit = 0;

        //--------------------
        // Wlan_INSTALLED
        //--------------------
        if( itemid == WL_INSTALLED )
        {
            UseWL = Config.UseWL;
            Config.UseWL = Edit_generic(Config.UseWL,0,1,WL1,YesNo,1);

            if( UseWL != Config.UseWL )                 // hat Benutzer Einstellung geaendert?
            {
                if( Config.UseWL )                      // Wlan wurde aktiviert - initialisieren
                {
                    if( wl_init() )
                        Config.WLIsSet = true;
                    else
						Config.WLIsSet = false;
                }

                MenuCtrl_Destroy();                     // Menue aendern wegen wechsel Wlan vorhanden / nicht vorhanden
                Setup_WiFly_MenuCreate();
                continue;
            }
        }

        //--------------------
        // WL_SSID
        //--------------------
        if( itemid == WL_SSID )
        {
            strncpyfill( string, Config.Wlan_SSID,wlan_ssid_length+1 );                    // wlan_ssid_length
            Edit_String( string, wlan_ssid_length , EDIT_SSID, PSTR("      Wlan SSID      "));

            if( edit == 1 )
            {
                strrtrim( EditString);                                                  // Leerzeichen rechts entfernen
                strncpy( Config.Wlan_SSID, EditString, wlan_ssid_length+1 );
            }
        }

        //--------------------
        // Wlan_PASSWORD
        //--------------------
        if( itemid == WL_PASSWORD)
        {
            strncpyfill( string, Config.Wlan_Password,wlan_password_length+1 );                    //
            Edit_String( string, wlan_password_length , EDIT_WL_PASSWORD, PSTR("    Wlan Password    "));

            if( edit == 1 )
            {
                strrtrim( EditString);                                                  // Leerzeichen rechts entfernen
                strncpy( Config.Wlan_Password, EditString, wlan_password_length+1 );
            }
        }

        //--------------------
        // Wlan_Channel
        //--------------------
        if( itemid == WL_CHANNEL )
        {
			Config.Wlan_Channel = Edit_generic(Config.Wlan_Channel,0,13,WL4,Show_uint3,1);
        }

        //--------------------
        // Wlan_INIT
        //--------------------
        if( itemid == WL_INIT )
        {
            lcd_cls ();
            Old_Baudrate = Config.PKT_Baudrate;
            SetBaudUart1(Config.PKT_Baudrate);
			lcd_printp_at( 0, 0, PSTR("      Wlan Init      "), MINVERS);
            if( wl_init() == true )
            {
                lcd_printp_at( 0, 3, PSTR("Wlan Init Ok"), MNORMAL);
                WriteWLInitFlag();
            }
            else
            {
                lcd_printp_at( 0, 3, PSTR("Wlan Init Error"), MNORMAL);
                Config.WLIsSet = false;
                set_beep( 1000, 0x0040, BeepNormal);
            }
            _delay_ms(4000);
            SetBaudUart1(Old_Baudrate);
        }
		
        //--------------------
         // Wlan_RESET
         //--------------------
         if( itemid == WL_RESET )
         {
            lcd_cls ();
            Old_Baudrate = Config.PKT_Baudrate;
            SetBaudUart1(Config.PKT_Baudrate);
			lcd_printp_at( 0, 0, PSTR("      Wlan Reset     "), MINVERS);
            if( wl_reset() == true )
            {
	            lcd_printp_at( 0, 3, PSTR("Wlan Reset Ok"), MNORMAL);
	            WriteWLInitFlag();
            }
            else
            {
	            lcd_printp_at( 0, 3, PSTR("Wlan Reset Error"), MNORMAL);
	            Config.WLIsSet = false;
	            set_beep( 1000, 0x0040, BeepNormal);
            }
            _delay_ms(4000);
            SetBaudUart1(Old_Baudrate);
         }
		 
		 //--------------------
		 // Wlan_VERSION
		 //--------------------
		 if( itemid == WL_VERSION )
		 {
			 lcd_cls();
			 Old_Baudrate = Config.PKT_Baudrate;
			 SetBaudUart1(Config.PKT_Baudrate);
			 lcd_printp_at( 0, 0, PSTR("     Wlan Version    "), MINVERS);
			 lcd_puts_at( 5, 3, getVersion(buf, sizeof(buf)), MNORMAL);
			 lcd_printp_at( 0, 3, PSTR("Ver:"), MNORMAL);
			 
			 _delay_ms(4000);
			 SetBaudUart1(Old_Baudrate);
		 }
		 
		 //--------------------
		 // Wlan_UPDATE
		 //--------------------
		 if( itemid == WL_UPDATE )
		 {
			lcd_cls ();
			Old_Baudrate = Config.PKT_Baudrate;
			SetBaudUart1(Config.PKT_Baudrate);
			lcd_printp_at( 0, 0, PSTR("     Wlan Update     "), MINVERS);
			if( wl_update() == true )
			{
				lcd_printp_at( 0, 3, PSTR("Wlan Update Ok"), MNORMAL);
				WriteWLInitFlag();
			}
			else
			{
				lcd_printp_at( 0, 3, PSTR("Wlan Update Error"), MNORMAL);
				Config.WLIsSet = false;
				set_beep( 1000, 0x0040, BeepNormal);
			}
			_delay_ms(4000);
			SetBaudUart1(Old_Baudrate);	 
		 }
		 
		 //--------------------
         // Wlan_WPA
         //--------------------
//		 if( itemid == WL_WPA )
//		 {
//			Config.Wlan_WPA = Edit_generic(Config.Wlan_WPA,0,1,WL3,WlanSecurity,1);
//		 }		 		

    }  // end: while( true )

    //---------------
    // Destroy
    //---------------
    MenuCtrl_Destroy();

} // end: Setup_WiFly()

#endif
